@@ -11,6 +11,35 @@ from dl2050utils.ju import in_ipynb
 from IPython.display import display
 T = torch.tensor
 
+def get_logprobs(model, x):
+    x = torch.Tensor(x).cuda()
+    with torch.no_grad():
+        y2 = model(x)
+    return y2.detach().cpu().numpy()
+
+def get_probs_from_logprobs(y2):
+    return np.exp(np.max(y2, axis=1))
+    return np.exp(y2[:,1])
+
+def get_preds_from_logprobs(y2):
+    return y2.argmax(axis=1)
+
+def get_probs(model, x):
+    y2 = get_logprobs(model, x)
+    return get_probs_from_logprobs(y2)
+
+def get_preds(model, x):
+    y2 = get_logprobs(model, x)
+    return get_preds_from_logprobs(y2)
+
+def predict(model, x):
+    x = torch.Tensor(x).cuda()
+    with torch.no_grad():
+        y2 = model(x)
+    y2 = y2.detach().cpu().numpy()
+    preds = y2.argmax(axis=1)
+    return preds
+
 def norm(x):
     m,s = x.mean(), x.std()
     return (x-m)/s if s>0 else x
@@ -64,7 +93,6 @@ def np_split(x, y, p=.8):
     x,y = x[idx].copy(),y[idx].copy()
     return x[:k],y[:k],x[k:],y[k:]
 
-
 def get_sample(x1, y1, x2, y2, sz=1000, sz2=None):
     if sz2 is None: sz2=sz
     p = np.random.permutation(len(x1))
@@ -158,15 +186,22 @@ def save_model(model, path, stats):
     state = {'state_dict': model.state_dict(), 'stats': stats}
     torch.save(state, path)
 
-
-def load_model(model, path):
-    state = torch.load(path,  map_location='cpu')
+def load_model(model, path, cuda=False, inference=False, show=False):
+    state = torch.load(path, map_location='cpu')
     model.load_state_dict(state['state_dict'])
-    return state['stats']
+    stats = state['stats']
+    if show:
+        pd.options.display.float_format = '{:,.4f}'.format
+        display(pd.DataFrame.from_dict(stats))
+    if cuda:
+        model.cuda()
+    if inference:
+        model.eval()
+    return stats
 
 
-def get_model_n_params(model): return sum([p.numel() for p in model.parameters()])
-
+def get_model_n_params(model):
+    return sum([p.numel() for p in model.parameters()])
 
 def xy_shuffle_split(x, y, p=0.2):
     msk = np.random.permutation(len(x))
