@@ -1,4 +1,6 @@
 import torch
+from functools import partial
+import shutil
 from dl2050nn.cbs import CancelBatchException, CancelEpochException, CancelTrainException, LR_Find, Recorder
 from dl2050nn.optimizers import *
 from dl2050nn.log import LoggerCallback
@@ -6,15 +8,14 @@ from dl2050nn.etc import *
 
 class Learner():
 
-    def __init__(self, data, model, opt=Adam, loss_f=None, metrics=[], cbs=[], device=None, name=None, path='.', save=True, load=False, plot=True):
+    def __init__(self, data, model, opt=Adam, loss_f=None, metrics=[], cbs=[], device=None, model_name=None, pre=None, path='/data', save=True, show=True):
         if not path.is_dir(): raise Exception()
         if not (path/'models').exists(): (path/'models').mkdir()
-        self.name, self.path = name if name is not None else str(model.__class__.__name__), path
-        self.path_model = path/'models'/f'{name}.pth.tar'
+        self.model = model
+        self.model_name,self.path = model_name or str(model.__class__.__name__),path
         self.data = data
         self.device = 'cpu' if not torch.cuda.is_available() else torch.device(0) if device is None else torch.device(device)
         print(f'Learner: device={self.device}\nBatch size: {data.bs}')
-        self.model = model.to(self.device)
         self.opt = opt
         self.optf = self.opt(model.parameters())
         self.loss_f = loss_f
@@ -22,11 +23,32 @@ class Learner():
         self.metrics = metrics
         self._init_cbs([LoggerCallback]+cbs)
         self.ep, self.train = 0, False
-        self._save, self._plot = save, plot
-        if load: self.load()
+        self._save,self._plot = save,show
+        if pre:
+            self.load(model_name=pre, update_best=False, show=show)
+        self.model.to(self.device)
 
     @property
     def df_best(self): return self.logger.df_best
+
+    def load(self, model_name=None, update_best=True, show=True):
+        p = self.path/'models'/f'{model_name or self.model_name}.pth.tar'
+        stats = load_model(self.model, p, show=show)
+        if not update_best:
+            return
+        # self.eval()
+        self.logger.df_best = pd.DataFrame.from_dict(stats)
+        k = 4+len(self.metrics)
+        self.logger.best = self.logger.df_best.iloc[0,k]
+
+    def save(self, model_name=None):
+        p = self.path/'models'/f'{model_name or self.model_name}.pth.tar'
+        save_model(self.model, p, self.logger.df_best.to_dict())
+
+    def backup_model(self, backup_name):
+        p = self.path/f'models/{backup_name}.pth.tar'
+        shutil.copy(self.path/f'models/{self.model_name}.pth.tar', p)
+        print(f'Model backup up: {p}')
     
     def _init_cbs(self, cbs):
         self.cbs = []
@@ -108,26 +130,6 @@ class Learner():
         with torch.no_grad(): 
             if not self('begin_validate'): self.all_batches(self.data.dl2)
 
-    def save(self, name=None):
-        path_model = self.path_model if name is None else self.path/'models'/f'{name}.pth.tar'
-        save_model(self.model, path_model, self.logger.df_best.to_dict())
-
-    def load(self, name=None, mute=False, update_best=True):
-        path_model = self.path_model if name is None else self.path/'models'/f'{name}.pth.tar'
-        self.logger.best = 0.0 if self.logger.maximize else 1e9
-        self.logger.df_best = None
-        stats = load_model(self.model, path_model)
-        df_best = pd.DataFrame.from_dict(stats)
-        if not mute:
-            print(f'=> model loaded {path_model}')
-            # display(df_best)
-        if update_best:
-            self.logger.df_best = df_best
-            k = 4+len(self.metrics)
-            self.logger.best = self.logger.df_best.iloc[0,k]
-            self.eval()
-            if not mute: self.logger.show_best()
-
     def predict(self, x):
         x = self.data.encode(x)
         self.model.eval()
