@@ -189,8 +189,10 @@ def get_predictions_clsf_tabular(data, model, df, device='cpu'):
     return df
 
 class Results_Clsf():
-    def __init__(self, learner):
-        y2,y = learner.logger.y2_ep.cpu().numpy(), learner.logger.y_ep.cpu().numpy()
+    def __init__(self, learner=None, y2=None, y=None):
+        assert learner is not None or (y2 is not None and y is not None), 'required either learner or y2 and y'
+        if learner is not None:
+            y2,y = learner.logger.y2_ep.cpu().numpy(),learner.logger.y_ep.cpu().numpy()
         self.actuals = y
         self.c = len(np.unique(y))
         self.probs = get_probs_bin(y2) if self.c==2 else get_probs(y2)
@@ -203,6 +205,7 @@ class Results_Clsf():
         if self.c!=2: raise ValueError('More than two classes')
         cm,cm_p = calc_cm(self.c, self.actuals, self.preds)
         plot_confusion(cm, cm_p)
+        display(pd.DataFrame([{'Total':cm.sum()}]))
         display(pd.DataFrame(cm.sum(axis=0)).transpose())
         display(pd.DataFrame(cm.sum(axis=1)))
         display(df_stats_by_class(cm))
@@ -213,14 +216,19 @@ class Results_Clsf():
     def roc(self):
         if self.c!=2: raise ValueError('More than two classes')
         plot_roc(self.actuals, self.probs)
-    def best(self, n=8, **kwargs): return np.argsort(self.probs)[::-1][:n]
-    def worst(self, n=8, **kwargs): return np.argsort(self.probs)[:n]
-
-# y2,y = learner.logger.y2_ep.cpu().numpy(), learner.logger.y_ep.cpu().numpy()
-# c = len(np.unique(actuals))
-# probs = get_probs_bin(y2) if c==2 else get_probs(y2)
-# actuals = y
-# preds = get_preds(probs)
+    def get_wrong(self, actual=None, n=8):
+        idx = np.where(self.actuals!=self.preds)[0]
+        if actual is not None:
+            idx = [i for i in idx if self.actual[i]==actual]
+        return idx[:n]
+    def get_most_confident(self, n=8):
+        return np.argsort(np.absolute(self.probs-.5))[::-1][:n]
+    def get_least_confident(self, n=8):
+        return np.argsort(np.absolute(self.probs-.5))[:n]
+    def best(self, n=8):
+        return np.argsort(self.probs)[::-1][:n]
+    def worst(self, n=8):
+        return np.argsort(self.probs)[:n]
 
 class Results_Clsf_Tabular(Results_Clsf):
     def __init__(self, data, df):
